/* nnanalysis3.c --- Matlab Interface to the nnanalysis routine */ 

 /* Dan Lofgreen 12 March 2007
 *
 * This software is distributed free under the conditions that
 * (1) it shall not be incorporated in software that is subsequently sold
 * (2) the authorship of the software shall be acknowledged in any 
 *     publication that uses results generated by the software
 * (3) this notice shall remain in place in each file.
 *
 * GPL applies: http://www.gnu.org/copyleft/gpl.html
 */ 

/* Windows compile:
 * mex -v nnanalysis3.c triarea3.c tetraarea.c
 */

/* Arguments:
 *
 * Matlab calling function:
 * [SegLen,FacetArea,TotalArea,Vol,SegIndex,FacetIndex,NumFacets,MinNN,
 *  MaxNN,AvgNN,NumNN,NNIndices,NNIndex] = nnanalysis(T,p)
 *
 * T            Delaunay tesselation
 * p            Points from the tesselation
 *
 * SegLen       Length of each connector for each tetrahedron (6 per)
 * FacetArea    Area of each facet for each tetrahedron (4 per)
 * TotalArea    Total surface area of each tetrahedron
 * Vol          Volume of each tetrahedron in the tessellation
 * SegIndex     Indices of the connectors for each tetrahedron
 * FacetIndex   Indices of the facets for each tetrahedron
 * NumFacets    Number of facets each point is associated with
 * MinNN        Minimum near neighbor distance
 * MaxNN        Maximum near neighbor distance
 * AvgNN        Average near neighbor distance
 * NumNN        Total number of near neighbors
 * NNIndices    A matrix that contains the indices of each point's near neighbors
 * NNIndex      A vector containing the index of the nearEST neighbor point
 *
 */

#include "mex.h"
#include "matrix.h"
#include "SpatialAnalysis3.h"
#include <math.h>

void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* Initialize all the variables */
    int i,j,a,b,A,npts,ntet;
    double s1[3], s2[3], n[3], D, h, norm, si[6], Ai[4];
    double *x1, *y1, *z1, *T, *p, *p1, *p2, *p3, *p4;
    double x[4], y[4], z[4];
    double *Vol, *TotalArea, *FacetArea , *SegLen, *FacetIndex, *SegIndex;
    double *NumFacets, *MinNN, *MaxNN, *AvgNN, *NumNN;
    double *NNIndices, *NNIndex, newdist, inf;
  
    /* Check to see if the user passed the correct number of arguments to 
     * to the function and asked for the right number to be returned */
    if (nrhs != 2 ) 
        mexErrMsgTxt("Two arguments please on RHS.");
        
    if (nlhs != 13) 
        mexErrMsgTxt("Thirteen arguments please on LHS.");
    
    /* Get the pointers to the data that was passed, T and p */
    T = mxGetPr(prhs[0]);
    p = mxGetPr(prhs[1]);
    
    /* Get the number of Delaunay tessellations and points*/
    ntet = mxGetM(prhs[0]);
    npts = mxGetM(prhs[1]);

    /* Create a matrix for the segment lengths (6 per tetrahedron) */
    plhs[0] = mxCreateDoubleMatrix(6*ntet, 1, mxREAL);
    SegLen = mxGetPr(plhs[0]);    

    /* Create a matrix for the facet areas (4 per tetrahedron) */
    plhs[1] = mxCreateDoubleMatrix(4*ntet, 1, mxREAL);
    FacetArea = mxGetPr(plhs[1]);    

    /* Create the array for the total surface area of each tetrahedron */
    plhs[2] = mxCreateDoubleMatrix(ntet, 1, mxREAL);
    TotalArea = mxGetPr(plhs[2]);
    
    /* Create the array for the Volume of each tetrahedron */
    plhs[3] = mxCreateDoubleMatrix(ntet, 1, mxREAL);
    Vol = mxGetPr(plhs[3]);
    
    /* Create a matrix for the segment indices (2 per segment) */
    plhs[4] = mxCreateDoubleMatrix(6*ntet, 2, mxREAL);
    SegIndex = mxGetPr(plhs[4]);    
    
     /* Create a matrix for the facet indices (3 per facet) */
    plhs[5] = mxCreateDoubleMatrix(4*ntet, 3, mxREAL);
    FacetIndex = mxGetPr(plhs[5]);    
   
    /* Create the array for the Number of facets */
    plhs[6] = mxCreateDoubleMatrix(npts, 1, mxREAL);
    NumFacets = mxGetPr(plhs[6]);
    
    /* Create the array for the minimum NN distance */
    plhs[7] = mxCreateDoubleMatrix(npts, 1, mxREAL);
    MinNN = mxGetPr(plhs[7]);

    /* Create the array for the maximum NN Distance */
    plhs[8] = mxCreateDoubleMatrix(npts, 1, mxREAL);
    MaxNN = mxGetPr(plhs[8]);

    /* Create the array for the average NN distance */
    plhs[9] = mxCreateDoubleMatrix(npts, 1, mxREAL);
    AvgNN = mxGetPr(plhs[9]);

    /* Create an array for the number of near neighbors */
    plhs[10] = mxCreateDoubleMatrix(npts, 1, mxREAL);
    NumNN = mxGetPr(plhs[10]);

    /* Create the array for the near neighbor indices */
    plhs[11] = mxCreateDoubleMatrix(npts, npts, mxREAL);
    NNIndices = mxGetPr(plhs[11]);

    /* Create the array for the nearEST neighbor indices */
    plhs[12] = mxCreateDoubleMatrix(npts, 1, mxREAL);
    NNIndex = mxGetPr(plhs[12]);

    /* Get the maximum value allowed on the system so we can initialize
     * the minimum arrays to this value.  That way we'll be assured that
     * we always find the minimum */
    inf = mxGetInf();
    
    /* Intialize the minimum, maximum and NumFacets arrays */
    for(i=0; i<npts; i++)
    {
        NumFacets[i] = 0;
        MinNN[i] = inf;
        MaxNN[i] = 0;
        AvgNN[i] = 0;
        NumNN[i] = 0;
        TotalArea[1] = 0;
    }
    
    /* Create pointers to the x, y, and z points */
    x1 = p;
    y1 = &p[npts];
    z1 = &p[2*npts];
    
    /* There are 4 points to a tetrahedron. Create a pointer for each point 
     Remember that T contains indices into p that begin at 1 (Matlab) not at
     0 (C or C++) */
    p1 = T;
    p2 = &T[ntet];
    p3 = &T[2*ntet];
    p4 = &T[3*ntet];
    
    
/* Go through each tetrahedron and calculate the facet area and volume */
    for(i=0; i<ntet; i++)
    {
        /* Build the x, y and z vectors for the 4 points */
        for(a=0; a<4; a++)
        {
            x[a] = x1[(int)T[i+ntet*a]-1];
            y[a] = y1[(int)T[i+ntet*a]-1];
            z[a] = z1[(int)T[i+ntet*a]-1];
        }

        /* This defines two vectors in the plane, s1 & s2 */
        s1[0] = x[1]-x[0];
        s1[1] = y[1]-y[0];
        s1[2] = z[1]-z[0];
        
        s2[0] = x[2]-x[0];
        s2[1] = y[2]-y[0];
        s2[2] = z[2]-z[0];
        
        /* Is the normal vector to the plane */
        n[0] = s1[1]*s2[2]-s1[2]*s2[1];
        n[1] = s1[2]*s2[0]-s1[0]*s2[2];
        n[2] = s1[0]*s2[1]-s1[1]*s2[0];
        
        /* The constant term is calculated from a point in the plane */
        D = -(n[0]*x[0] + n[1]*y[0] + n[2]*z[0]);
        
        /* Now calculate the height */
        norm = sqrt(n[0]*n[0] + n[1]*n[1] + n[2]*n[2]);
        h = n[0]*x[3] + n[1]*y[3] + n[2]*z[3];
        h = fabs((h+D)/norm);

        /* Calculate the total area of the tetrahedron.  This function also
         * calculates the 6 segments lengths and 4 facet areas of the 
         * tetrahedron and puts them in si and Ai respectively.  The "base"
         * of the tetrahedron is in Ai[0] */
        tetraarea(x,y,z,si,Ai);
      
        /* Calculate the volume */
        Vol[i] = 1.0/3.0*Ai[0]*h;
        
        /* Load the segment length data into the matrix. WE MIGHT ACTUALLY 
         * BE ABLE TO DO THE NEAREST NEIGHBOR CALCULATIONS HERE */
        for(a=0;a<6;a++)
            SegLen[i*6 + a] = si[a];

        /* Remember all the indices that were used to build the seg lengths */
        /* First segment indices */
        SegIndex[i*6] = p1[i];
        SegIndex[(i+ntet)*6] = p2[i];
        /* Second segment indices */
        SegIndex[i*6+1] = p2[i];
        SegIndex[(i+ntet)*6+1] = p3[i];
        /* Third segment indices */
        SegIndex[i*6+2] = p1[i];
        SegIndex[(i+ntet)*6+2] = p3[i];
        /* Fourth segment indices */
        SegIndex[i*6+3] = p1[i];
        SegIndex[(i+ntet)*6+3] = p4[i];
        /* Fifth segment indices */
        SegIndex[i*6+4] = p2[i];
        SegIndex[(i+ntet)*6+4] = p4[i];
        /* Sixth segment indices */
        SegIndex[i*6+5] = p3[i];
        SegIndex[(i+ntet)*6+5] = p4[i];       
        
        /* Remember the indices that were used to build the facets */
        /* First Facet */
        FacetIndex[i*4] = p1[i];
        FacetIndex[(i+ntet)*4] = p2[i];
        FacetIndex[(i+2*ntet)*4] = p3[i];
        /* Second Facet */
        FacetIndex[i*4+1] = p1[i];
        FacetIndex[(i+ntet)*4+1] = p2[i];
        FacetIndex[(i+2*ntet)*4+1] = p4[i];
        /* Third Facet */
        FacetIndex[i*4+2] = p2[i];
        FacetIndex[(i+ntet)*4+2] = p3[i];
        FacetIndex[(i+2*ntet)*4+2] = p4[i];
        /* Fourth Facet */
        FacetIndex[i*4+3] = p1[i];
        FacetIndex[(i+ntet)*4+3] = p3[i];
        FacetIndex[(i+2*ntet)*4+3] = p4[i];
        
        /* Each tetrahedron is made up of 4 points.  So, each of these 4 
         * points share this volume.  These 4 points also share 3 facets
         * each, 3 different facets of the 4 facets for each point of 
         * course */
        for(a=0; a<4; a++)
        {
            /* Since we already have a loop for the 4 points, we can load 
             * the Facet area here */
            FacetArea[i*4 + a] = Ai[a];
            TotalArea[i] = TotalArea[i]+Ai[a];
                        
            /* Increase the number of facets per point */
            NumFacets[(int)T[i+ntet*a]-1] = NumFacets[(int)T[i+ntet*a]-1]+1;
                     
            /* Each of the 4 points is a near neighbor of the other 3, but
             * not itself of course.  So, build a matrix that stores the
             * index number of the near neighbors on a row basis.  I.E. 
             * each row corresponds to a point of interest npts number of
             * rows, and each colum represents one neighbor, npt columns.
             * A zero in the column means that point is NOT a neighbor. */
            for(b=0; b<a; b++)
            {
                A = (T[i+ntet*a]-1+npts*(T[i+ntet*b]-1));
                NNIndices[(int)A] = T[i+ntet*b];
            }
            for(b=a+1; b<4; b++)
            {
                A = (T[i+ntet*a]-1+npts*(T[i+ntet*b]-1));
                NNIndices[(int)A] = T[i+ntet*b];
            }
        }
    }
/* Now that we have this matrix NNIndices that is a npt x npt matrix that
 * contains the indices for the near neighbors we can calculate the 
 * nearEST neighbor distance, the maximum distance, the average distance,
 * and the number of near neighbors.  The matrix is set up like this:
 *
 *  |0  2  3  0  0 ... |   Where a number in a certain column means that
 *  |1  0  0  4  0 ... |   index is a near neighbor of the particular
 *  |:  :              |   row index.
 *
 *  In the example above, Point #1 (row 1)  has near neighbors Points
 *  2 and 3.  Point #2 (row 2) has near neighbors Points 1 and 4    */

    /* Go through each point now, and calculate the nearest, farthest, 
     * and average distance, while keeping track of the number of neighbors */
    
    for(i=0; i<npts; i++)
    {
        for(j=0;j<npts;j++)
        {
            A = NNIndices[i+j*npts];
            
            /* Check to see if we have a value other than 0 in A */
            if(A)
            {
                /* Calculate the distance of this point to the point of
                 * interest, and increase the number of neighbors */
                NumNN[i] = NumNN[i]+1;
                newdist = sqrt((x1[i]-x1[A-1])*(x1[i]-x1[A-1]) + 
                   (y1[i]-y1[A-1])*(y1[i]-y1[A-1]) +(z1[i]-z1[A-1])*(z1[i]-z1[A-1]));
                
                /* Check to see if this distance is less than the Min NN 
                 * distance */
                if(newdist<MinNN[i])
                {
                    MinNN[i] = newdist;
                    NNIndex[i] = A;
                }
                
                /* Check to see if this distance is more than the Max NN
                 * distance */
                if(newdist>MaxNN[i])
                    MaxNN[i] = newdist;
                
                /* Add this distance to the average distance, we will 
                 * divide by the total number of neighbors later */
                AvgNN[i] = AvgNN[i] + newdist;
                
            } /* End of the if statement */
        } /* End of the j for loop */
        
        /* Now that we are moving on to the next point, we can divide the 
         * Average NN distance by the number of NN to get average */
        AvgNN[i] = AvgNN[i]/NumNN[i];
        
    } /* End of the i for loop */
} /* End of the function */
    





